; scale_pattern.scm
; by Rob Antonishen
; http://ffaat.pointclark.net

; Version 1.2 (20090413)
; Version 1.3 (20190404)

; Description
; Scales the selected pattern and puts it in the clipboard to allow it to be used as a pattern
;
; Changes
; 1.1 - Added interactive calls to sharpen the pattern.
; 1.2 - If larger than 512x512 will save it, refresh patterns, and set the active pattern to be the saved file.
; 1.3 - Adaptation to Gimp 2.10, added Rotate (attention: not all patterns on the 90 or 270 degree rotation will remain seamless), added Transform-flip, add choosing your own criterion for writing to disk (by MareroQ)
; 1.3.de Deutsche Übersetzung by gimplyworxs@gimp-werkstett.de 11/2021

; License:
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; The GNU Public License is available at
; http://www.gnu.org/copyleft/gpl.html

(define (script-fu-scale_pattern inScale name save inSharpen inRotate inTransform)
(let* 
  (
    (inScale (/ inScale 100))
    (width (car (gimp-pattern-get-info (car(gimp-context-get-pattern)))))
    (height (cadr (gimp-pattern-get-info (car(gimp-context-get-pattern)))))
    ;(tempimage (car (gimp-image-new (* 3 width) (* 3 height) RGB)))
	; Gimp 2.10: RGB-IMAGE (0), RGBA-IMAGE (1), GRAY-IMAGE (2), GRAYA-IMAGE (3), INDEXED-IMAGE (4), INDEXEDA-IMAGE (5)
    (tempimage (car (gimp-image-new (* 3 width) (* 3 height) 0)))	; MrQ

    ;(templayer (car (gimp-layer-new tempimage (* 3 width) (* 3 height) RGBA-IMAGE "temp" 100 NORMAL-MODE)))
	; Gimp 2.10: LAYER-MODE-NORMAL-LEGACY (0)
    (templayer (car (gimp-layer-new tempimage (* 3 width) (* 3 height) 1 "temp" 100 0)))	; MrQ
    (filename (string-append gimp-directory DIR-SEPARATOR "patterns" DIR-SEPARATOR name))
  )

  ;(gimp-image-add-layer tempimage templayer -1) deprecated
  (gimp-image-insert-layer tempimage templayer 0 -1)
  
  ;(gimp-drawable-fill templayer PATTERN-FILL) deprecated PATTERN-FILL)FILL-FOREGROUND (0), FILL-BACKGROUND (1), FILL-WHITE (2), FILL-TRANSPARENT (3), FILL-PATTERN (4)
  (gimp-drawable-fill templayer 4)
  ;(gimp-image-scale-full tempimage (* width inScale 3) (* height inScale 3) INTERPOLATION-LANCZOS) 
  (gimp-image-scale tempimage (* width inScale 3) (* height inScale 3))  ; MrQ

  (if (= inSharpen 1)
    (plug-in-sharpen 0 tempimage templayer 10)
  )
  (if (= inSharpen 2)
    (plug-in-unsharp-mask 0 tempimage templayer 1 3 0)
  )	
  (if (= inRotate 1)   ; rotate add MrQ Angle  90 (1), 180 (2), 270 (3)
 	(plug-in-rotate 1 tempimage templayer 1 1)
  )
  (if (= inRotate 2)
 	(plug-in-rotate 1 tempimage templayer 2 1)
  )
   (if (= inRotate 3)
 	(plug-in-rotate 1 tempimage templayer 3 1)
  )
   (if (= inTransform 1)		; MrQ ORIENTATION-HORIZONTAL (0), ORIENTATION-VERTICAL (1) 
 	(gimp-item-transform-flip-simple templayer 0 1 0 )
  )
   (if (= inTransform 2)
 	(gimp-item-transform-flip-simple templayer 1 1 0 )
  )   
  ;(if (and (< (* width inScale) 512) (< (* height inScale) 512))
  (if (and (< (* width inScale) save) (< (* height inScale) save))  ; MrQ criterion of writing to disk
    (begin
      ;(gimp-rect-select tempimage (* width inScale) (* height inScale) (* width inScale) (* height inScale) CHANNEL-OP-REPLACE FALSE 0)
      (gimp-image-select-rectangle tempimage CHANNEL-OP-REPLACE (* width inScale) (* height inScale) (* width inScale) (* height inScale))	  
      (gimp-edit-copy templayer)
      (gimp-context-set-pattern (list-ref (cadr (gimp-patterns-get-list "")) 0))
    )
    (begin
      (gimp-image-crop tempimage (* width inScale) (* height inScale) (* width inScale) (* height inScale))
      (file-pat-save RUN-NONINTERACTIVE tempimage templayer filename name name)
     	(gimp-patterns-refresh)
      (gimp-context-set-pattern name)
    )
  )
  (gimp-image-delete tempimage)
)
)
    

;(script-fu-register "script-fu-scale_pattern"
          ;"Scale-Rotate Pattern..."
          ;"Scales/rotate the current pattern and make it the active pattern."
          ;"Rob Antonishen/MrQ"
          ;"RobA by license GPL"
          ;"Jan 2009/2019"
          ;""
          ;SF-ADJUSTMENT  "Pattern Scale %"          (list 100 5 500 1 10 0 1)
		  ;SF-STRING      "Name Pattern (*.pat)"               "Gimp.pat"		  
		  ;SF-ADJUSTMENT  "Save it if larger than\n(rescaled pattern in pix)" (list 256 5 1500 1 10 0 1)
          ;SF-OPTION      "Sharpening" (list "None" "Sharpen" "Unsharp Mask")
		  ;SF-OPTION      "Rotate (angle)" (list "None" " 90° " " 180° " " 270° ")
		  ;SF-OPTION      "Transform-flip" (list "None" " ORIENTATION-HORIZONTAL " " ORIENTATION-VERTICAL ")		  
;)

; deutsch
(script-fu-register "script-fu-scale_pattern"
          "Pattern Drehen/ Skalieren..."
          "Skaliert/dreht das aktuelle Muster und macht es zum aktiven Muster."
          "Rob Antonishen/MrQ"
          "RobA by license GPL"
          "Jan 2009/2019"
          ""
          SF-ADJUSTMENT  "Pattern Skalierung %"          (list 100 5 500 1 10 0 1)
		  SF-STRING      "Name für das neue Pattern (*.pat)"               "Gimp.pat"		  
		  SF-ADJUSTMENT  "Speichere es, wenn es größer ist als\n (neu skaliertes Muster in Pixeln)" (list 256 5 1500 1 10 0 1)
          SF-OPTION      "Schärfen?" (list "Nein" "Schärfen" "Unscharfe Maske")
		  SF-OPTION      "Drehen (Winkel)" (list "Nichts" " 90° " " 180° " " 270° ")
		  SF-OPTION      "Spiegeln?" (list "Nichts" " ORIENTIERUNG-HORIZONTAL " " ORIENTIERUNG-VERTIKAL ")		  
)

(script-fu-menu-register "script-fu-scale_pattern" "<Patterns>")
